# Class: ssh::server::config
#
# Provide the configuration parameters necessary to configure sshd(8)
# through sshd_config(5) from a template.
#
class ssh::server::config (
  $acceptenv                       = undef,
  $addressfamily                   = undef,
  $allowagentforwarding            = undef,
  $allowgroups                     = undef,
  $allowstreamlocalforwarding      = undef,
  $allowtcpforwarding              = undef,
  $allowusers                      = undef,
  $authenticationmethods           = undef,
  $authorizedkeyscommand           = undef,
  $authorizedkeyscommanduser       = undef,
  $authorizedkeysfile              = undef,
  $authorizedprincipalscommand     = undef,
  $authorizedprincipalscommanduser = undef,
  $authorizedprincipalsfile        = undef,
  $banner                          = undef,
  $casignaturealgorithms           = undef,
  $challengeresponseauthentication = undef,
  $chrootdirectory                 = undef,
  $ciphers                         = undef,
  $clientalivecountmax             = undef,
  $clientaliveinterval             = undef,
  $compression                     = undef,
  $denygroups                      = undef,
  $denyusers                       = undef,
  $disableforwarding               = undef,
  $exposeauthinfo                  = undef,
  $fingerprinthash                 = undef,
  $forcecommand                    = undef,
  $gatewayports                    = undef,
  $gssapiauthentication            = undef,
  $gssapicleanupcredentials        = undef,
  $gssapikeyexchange               = undef,
  $gssapistorecredentialsonrekey   = undef,
  $gssapistrictacceptorcheck       = undef,
  $hostbasedacceptedkeytypes       = undef,
  $hostbasedauthentication         = undef,
  $hostbasedusesnamefrompacketonly = undef,
  $hostcertificate                 = undef,
  $hostkey                         = undef,
  $hostkeyagent                    = undef,
  $hostkeyalgorithms               = undef,
  $ignorerhosts                    = undef,
  $ignoreuserknownhosts            = undef,
  $ipqos                           = undef,
  $kbdinteractiveauthentication    = undef,
  $kerberosauthentication          = undef,
  $kerberosgetafstoken             = undef,
  $kerberosorlocalpasswd           = undef,
  $kerberosticketcleanup           = undef,
  $kexalgorithms                   = undef,
  $keyregenerationinterval         = undef,
  $listenaddress                   = undef,
  $logingracetime                  = undef,
  $macs                            = undef,
  $maxauthtries                    = undef,
  $maxsessions                     = undef,
  $maxstartups                     = undef,
  $passwordauthentication          = undef,
  $permitemptypasswords            = undef,
  $permitlisten                    = undef,
  $permitopen                      = undef,
  $permitrootlogin                 = 'no',
  $permittty                       = undef,
  $permittunnel                    = undef,
  $permituserenvironment           = undef,
  $permituserrc                    = undef,
  $pidfile                         = undef,
  $port                            = '22',
  $printlastlog                    = undef,
  $printmotd                       = undef,
  $protocol                        = '2',
  $pubkeyacceptedkeytypes          = undef,
  $pubkeyauthentication            = undef,
  $rdomain                         = undef,
  $rekeylimit                      = undef,
  $revokedkeys                     = undef,
  $rhostsrsaauthentication         = undef,
  $rsaauthentication               = undef,
  $saclsupport                     = undef,
  $serverkeybits                   = undef,
  $setenv                          = undef,
  $streamlocalbindmask             = undef,
  $streamlocalbindunlink           = undef,
  $strictmodes                     = undef,
  $syslogfacility                  = 'AUTH',
  $tcpkeepalive                    = undef,
  $trustedusercakeys               = undef,
  $useblacklist                    = undef,
  $usedns                          = undef,
  $uselogin                        = undef,
  $usepam                          = undef,
  $versionaddendum                 = undef,
  $x11displayoffset                = undef,
  $x11forwarding                   = undef,
  $x11uselocalhost                 = undef,
  $xauthlocation                   = undef,

  $log_level                = 'INFO', # underscore here because puppet
  $has_pam                  = false,
  $has_gssapi               = false,
) {
  include ssh

  $valid_keywords = [
    'AcceptEnv',
    'AddressFamily',
    'AllowAgentForwarding',
    'AllowGroups',
    'AllowStreamLocalForwarding',
    'AllowTcpForwarding',
    'AllowUsers',
    'AuthenticationMethods',
    'AuthorizedKeysCommand',
    'AuthorizedKeysCommandUser',
    'AuthorizedKeysFile',
    'AuthorizedPrincipalsCommand',
    'AuthorizedPrincipalsCommandUser',
    'AuthorizedPrincipalsFile',
    'Banner',
    'CASignatureAlgorithms',
    'ChallengeResponseAuthentication',
    'ChrootDirectory',
    'Ciphers',
    'ClientAliveCountMax',
    'ClientAliveInterval',
    'Compression',
    'DenyGroups',
    'DenyUsers',
    'DisableForwarding',
    'ExposeAuthInfo',
    'FingerprintHash',
    'ForceCommand',
    'GatewayPorts',
    'GSSAPIAuthentication',
    'GSSAPICleanupCredentials',
    'GSSAPIKeyExchange',
    'GSSAPIStoreCredentialsOnRekey',
    'GSSAPIStrictAcceptorCheck',
    'HostbasedAcceptedKeyTypes',
    'HostbasedAuthentication',
    'HostbasedUsesNameFromPacketOnly',
    'HostCertificate',
    'HostKey',
    'HostKeyAgent',
    'HostKeyAlgorithms',
    'IgnoreRhosts',
    'IgnoreUserKnownHosts',
    'IPQoS',
    'KbdInteractiveAuthentication',
    'KerberosAuthentication',
    'KerberosGetAFSToken',
    'KerberosOrLocalPasswd',
    'KerberosTicketCleanup',
    'KexAlgorithms',
    'KeyRegenerationInterval',
    'ListenAddress',
    'LoginGraceTime',
    #'LogLevel',
    'MACs',
    'MaxAuthTries',
    'MaxSessions',
    'MaxStartups',
    'PasswordAuthentication',
    'PermitEmptyPasswords',
    'PermitListen',
    'PermitOpen',
    'PermitRootLogin',
    'PermitTTY',
    'PermitTunnel',
    'PermitUserEnvironment',
    'PermitUserRC',
    'PidFile',
    'Port',
    'PrintLastLog',
    'PrintMotd',
    'Protocol',
    'PubkeyAcceptedKeyTypes',
    'PubkeyAuthentication',
    'RDomain',
    'RekeyLimit',
    'RevokedKeys',
    'RhostsRSAAuthentication',
    'RSAAuthentication',
    'SACLSupport',
    'ServerKeyBits',
    'SetEnv',
    'StreamLocalBindMask',
    'StreamLocalBindUnlink',
    'StrictModes',
    'SyslogFacility',
    'TCPKeepAlive',
    'TrustedUserCAKeys',
    'UseBlacklist',
    'UseDNS',
    'UseLogin',
    'UsePAM',
    'VersionAddendum',
    'X11DisplayOffset',
    'X11Forwarding',
    'X11UseLocalhost',
    'XAuthLocation',
  ]

  # Keywords that are joined by spaces in the presence of multiple
  # values
  $space_separated_keywords = [
    'AcceptEnv',
    'AuthorizedKeysFile',
    'DenyGroups',
    'DenyUsers',
  ]

  # Keywords that are configured with their own defined type.
  $defined_keywords = [
    'Match',
    'AllowGroups',
    'Subsystem',
  ]

  concat::fragment { 'sshd_config':
    order   => '10',
    target  => $ssh::sshd_config,
    content => template('ssh/sshd_config.erb'),
  }
}
